﻿using System;
using System.Linq;
using System.Data.Entity;
using System.Threading.Tasks;

namespace Demo_Async
{
    class Program
    {
        static void Main(string[] args)
        {
            //RealizarOperacoesDoBancodeDados();
            var tarefa = RealizarOperacoesDoBancodeDados();

            Console.WriteLine();
            Console.WriteLine("Frase do Dia:");
            Console.WriteLine(" Não se preocupe sobre se o fim do mundo vai ser hoje... ");
            Console.WriteLine(" Já é amanhã no Japão.");

            tarefa.Wait();

            Console.WriteLine();
            Console.WriteLine("Pressione algo para sair...");
            Console.ReadKey(); 
        }

        private static async Task<Blog> GetBlog()
        {
            Blog _blog = null;

            using (var context = new BloggingContext())
            {
                Console.WriteLine("Iniciando GetBlog...");

                _blog = await (context.Blogs.Where(s => s.BlogId == 1).FirstOrDefaultAsync<Blog>());

                Console.WriteLine("Terminando GetBlog...");

            }
            return _blog;
        }

        private static async Task SaveBlog(Blog blogAlterado)
        {

            using (var context = new BloggingContext())
            {
                context.Entry(blogAlterado).State = EntityState.Modified;
                Console.WriteLine("Iniciar SaveBlog...");
                int x = await (context.SaveChangesAsync());
                Console.WriteLine("Terminar SaveBlog...");
            }
        }

        public static void AsyncConsultaESalva()
        {
            var _blog = GetBlog();
            Console.WriteLine("Vamos fazer outra coisa até obtermos os blogs..");
            _blog.Wait();
            var blogSave = SaveBlog(_blog.Result);
            Console.WriteLine("Vamos fazer algo enquanto salvamos os blogs..");
            blogSave.Wait();
        }

        public static async Task RealizarOperacoesDoBancodeDados()
        {
            using (var db = new BloggingContext())
            {
                // Cria um novo blog e o salva
                db.Blogs.Add(new Blog
                {
                       Name = "Testando o Blog #" + (db.Blogs.Count() + 1)
                });

                //db.SaveChanges();
                Console.WriteLine("Chamando SaveChanges()");
                await db.SaveChangesAsync();
                Console.WriteLine("SaveChanges() terminou");

                // Consulta todos os blogs ordenados por nome
                var blogs = await  (from b in db.Blogs
                             orderby b.Name
                             select b).ToListAsync();

                // Exibe todos os blogs no Console 
                Console.WriteLine("Executando a consulta");
                Console.WriteLine();
                Console.WriteLine("Todos os blogs:");
                foreach (var blog in blogs)
                {
                    Console.WriteLine(" " + blog.Name);
                }
            }
        } 
    }
}
